/*
	This file is part of Nanos6 and is licensed under the terms contained in the COPYING file.
	
	Copyright (C) 2015-2017 Barcelona Supercomputing Center (BSC)
*/

#ifndef TASKLOOP_GENERATOR_HPP
#define TASKLOOP_GENERATOR_HPP


#include <nanos6.h>

#include "Taskloop.hpp"
#include "TaskloopInfo.hpp"

#include <InstrumentAddTask.hpp>
#include <InstrumentTaskId.hpp>
#include <InstrumentThreadInstrumentationContext.hpp>


class TaskloopGenerator {
public:
	static inline Taskloop* createCollaborator(Taskloop *parent)
	{
		assert(parent != nullptr);
		
		nanos6_task_info_t *taskInfo = parent->getTaskInfo();
		nanos6_task_invocation_info_t *taskInvocationInfo = parent->getTaskInvokationInfo();
		
		Instrument::task_id_t parentTaskId = parent->getInstrumentationTaskId();
		Instrument::ThreadInstrumentationContext instrumentationContext(parentTaskId);
		
		void *originalArgsBlock = parent->getArgsBlock();
		size_t originalArgsBlockSize = parent->getArgsBlockSize();
		
		Taskloop *taskloop = nullptr;
		void *argsBlock = nullptr;
		
		// In some cases, for instance in all Fortran codes, the args block is allocated
		// by a code generated by Mercurium. In those cases we must call to the duplicate
		// args block before actually creating the task.
		bool hasPreallocatedArgsBlock = parent->hasPreallocatedArgsBlock();
		if (hasPreallocatedArgsBlock) {
			assert(taskInfo->duplicate_args_block != nullptr);
			taskInfo->duplicate_args_block(originalArgsBlock, &argsBlock);
		}
		
		// Create the task for this partition
		nanos6_create_task(taskInfo, taskInvocationInfo, originalArgsBlockSize, (void **) &argsBlock, (void **) &taskloop, parent->getFlags(), 0);
		assert(argsBlock != nullptr);
		assert(taskloop != nullptr);
		
		// Copy the args block
		if (!hasPreallocatedArgsBlock) {
			if (taskInfo->duplicate_args_block != nullptr) {
				taskInfo->duplicate_args_block(originalArgsBlock, &argsBlock);
			} else {
				assert(!parent->hasPreallocatedArgsBlock());
				memcpy(argsBlock, originalArgsBlock, originalArgsBlockSize);
			}
		}
		
		// Set the flags
		taskloop->setRunnable(true);
		taskloop->setDelayedRelease(false);
		
		// Set the parent
		taskloop->setParent(parent);
		
		// Instrument the task creation
		Instrument::task_id_t taskInstrumentationId = taskloop->getInstrumentationTaskId();
		Instrument::createdTask(taskloop, taskInstrumentationId);
		Instrument::exitAddTask(taskInstrumentationId);
		
		return taskloop;
	}
};

#endif // TASKLOOP_GENERATOR_HPP
